#!/usr/bin/php

<?php

/**
 * This script can execute either an external arbitrary PHP file, or PHP script typed in the paramers.
 *
 * The script will convert variables passed with "Prompt for user input" to PHP variables that can be used in the script
 *
 * camel cased {variable name} / camel cased p{variable name}
 *
 * Any of the parameters are optional. If both are filled, the parameter for execute will determine which will run first
 *
 * If a PHP script is specified, the temporary folder is used (and at that point it is required)
 *
 * Version: 1.1
 * Author: Alon Sabi (alon@sabi.me)
 * Date: 2014-04-24
 * Last Updated: 2014-06-06
 * Last Updated By: Alon Sabi
 *
 */

$variableOrder = ini_get('variables_order');

// Sanity check to make sure the environment is setup properly

if (strpos($variableOrder, 'E') === FALSE) {
    echo "Sorry, this script requires the 'variables_order' to include the 'E' option\n";
    echo "You can adjust it by editing your /etc/php.ini file. 'EGPCS' should be the default value, which should work fine.\n";
    echo "Your variables_order is currently set to '$variableOrder' \n\n";
    echo "The script will now quit\n";
    exit;
}


//======================================================================
// Converts the Keyboard Maestro environment variables to PHP variables
//
//  $_ENV["KMPARAM_*"] becomes p{variable name} - Camel cased
//  $_ENV["KMVAR_*"] becomes v_{variable name} - Camel cased
//
//  Ex:
// 
//  $_ENV["KMPARAM_Execute"] becomes $pExecute
//  $_ENV["KMVAR_User Name"] becomes $userName
//
//======================================================================

$vars = array();

foreach ($_ENV as $key => $value) {

    if (strpos($key, 'KMPARAM_') !== false) {
        $varName = 'p_'.str_replace('KMPARAM_', '', $key);
        $vars[stringToCamelCase($varName)] = $value;
    }

    if (strpos($key, 'KMVAR_') !== false) {
        $varName = str_replace('KMVAR_', '', $key);
        $vars[stringToCamelCase($varName)] = $value;
 }
}

//======================================================================
// Executes the scripts in the order requested
//======================================================================


$fileScriptContent = executePHPfile($vars);
$inlinePhpScriptContent = executeInlinePHP($vars);



if ($vars['pExecute'] == 'File First') {
    echo $fileScriptContent.$inlinePhpScriptContent;
} else {
    echo $inlinePhpScriptContent.$fileScriptContent;
}

//======================================================================
// Helper functions
//======================================================================


/**
 * Executes the "raw" PHP from the PHP content variable. The function "dumps" the script into a temporary files,
 * and requires it. I found this easier than using the PHP Eval function (quotes / double quotes etc ...), also a bit easier
 * to debug if needed.
 *
 * @param $vars
 * @return string
 */
function executeInlinePHP($vars)
{
    $phpScriptOutput = '';


    // Convert the input variables into "full fledged" PHP variables in the local function scope
    foreach ($vars as $key => $value) {
        $$key = $value;
    }

    // for some reason, double quotes get the "special ones"
    $pOptionalInlinePHP = trim(str_replace(array("”", "“"), "\"", $pOptionalInlinePHP));

    if ($pOptionalInlinePHP != '') {
        if (!file_exists($pTemporaryFolder) || !is_writable($pTemporaryFolder)) {

            echo "\n=============================================================\n";
            echo "Note: the script needs to be able to write to the temporary folder provided ($p_Temporary_Folder).\n";
            echo "It is currently not writable, or does not exist\n";
            echo "=============================================================\n";

            exit;
        }


        $fileName = tempnam($pTemporaryFolder, 'php_script_');
        file_put_contents($fileName, $pOptionalInlinePHP);
        ob_start();
        require_once($fileName);
        $phpScriptOutput = ob_get_clean();
        unlink($fileName);
    }

    return $phpScriptOutput;

}

/**
 * Executes the provided file and returns the results.
 *
 * @param $vars
 * @return string
 */
function executePHPfile($vars) {

    $phpScriptOutput = '';


    // Convert the input variables into "full fledged" PHP variables in the local function scope
    foreach ($vars as $key => $value) {
        $$key = $value;
    }

    $pOptionalPHPFilePath = trim($pOptionalPHPFilePath);

    if ($pOptionalPHPFilePath != '' ) {

        if(!file_exists($pOptionalPHPFilePath)) {
            echo "Could not find the requested file: $pOptionalPHPFilePath\n";exit;
        }

        ob_start();
        require_once($pOptionalPHPFilePath);
        $phpScriptOutput = ob_get_clean();
    }

    return $phpScriptOutput;
}

function stringToCamelCase($string) 
{

    $str = str_replace(' ', '', ucwords(str_replace(array('-','_'), ' ', $string)));
    $str[0] = strtolower($str[0]);

    return $str;
}


  

